#define DATCTL 0xBC         // Data Control (data handling in RAM)
#define DISCTL 0xCA         // Display Control
#define GCP64 0xCB          // pulse set for 64 gray scale
#define GCP16 0xCC          // pulse set for 16 gray scale
#define OSSEL 0xD0          // Oscillator select
#define GSSET 0xCD          // set for gray scales
#define ASCSET 0xAA         // aerea scroll setting
#define SCSTART 0xAB        // scroll start setting
#define DISON 0xAF          // Display ON (no parameter)
#define DISOFF 0xAE         // Display OFF (no parameter)
#define DISINV 0xA7         // Display Invert (no parameter)
#define DISNOR 0xA6         // Display Normal (no parameter)
#define SLPIN  0x95         // Display Sleep (no parameter)
#define SLPOUT 0x94         // Display out of sleep (no parameter)
#define RAMWR 0x5C          // Display Memory write
#define PTLIN 0xA8          // partial screen write

#define SD_CSET 0x15        // column address setting
#define SD_PSET 0x75        // page address setting

#define DISP_W 132
#define DISP_H 176
#define CHAR_H 14
#define CHAR_W 8
#define TEXT_COL 16
#define TEXT_ROW 12

unsigned short textcolor=0xFFFF;    // color of text
unsigned short backcolor=0x00;    // color of background

#include "chartable.h"
#include "tools.h"
#include "lpc2103.h"
#include "l2f50126.h"

#define lcd_cs_high()		IOSET |= 0x80
#define lcd_cs_low()		IOCLR |= 0x80
#define lcd_reset_high()	IOSET |= 0x4
#define lcd_reset_low()		IOCLR |= 0x4
#define lcd_rs_high()		IOSET |= 0x8
#define lcd_rs_low()		IOCLR |= 0x8

void lcd_write(unsigned char dat)
{
  	while ( !(SSPSR & 0x02) );
	SSPDR = dat;
	while ( SSPSR & 0x10 );
}


void lcd_cmd(unsigned char dat)
{
  lcd_rs_low();

  lcd_write(dat);
  lcd_write(0x00);

  lcd_rs_high();
}

void lcd_dat0(unsigned char dat)
{
  lcd_write(dat);
  lcd_write(0x00);  
}


void lcd_dat16(unsigned short dat)
{
  lcd_write(dat>>8);
  lcd_write(dat);
}

void lcd_cspulse(void)
{
 	lcd_cs_high();
  	delay(10);
  	lcd_cs_low();
}


void lcd_init(void)
{
	unsigned int i;
	init_timer_delay();
	init_ssp();
	IODIR |= (1<<2); //P0_2; //RESET	Pull up
	IODIR |= (1<<3); //P0_3; //RS		Pull up
	IODIR |= (1<<7); //P0_7; //CS		Pull up
 /*
  Init the Siemens S65 Display with EPSON controller

  (c) Christian Kranz, 10/2005 modified by Steffen L. Graf, 6/2007
*/
   lcd_reset_low();
   lcd_cs_high();
   lcd_rs_high();
   delay(10000);
   lcd_reset_high();
   delay(35000);
   lcd_cs_low();

   lcd_cmd(DATCTL);
   lcd_dat0(0x2A);  // 0x2A=565 mode, 0x0A=666mode, 0x3A=444mode

   lcd_cspulse();
  
   lcd_cmd(DISCTL);
   for (i=0; i<9; i++)
   {
   		lcd_dat0(disctl[i]);
   }
  
  lcd_cmd(GCP64);
  for (i=0; i<29; i++)
  {
    lcd_dat0(gcp64_0[i]);
    lcd_dat0(0x00);
  }
  for (i=0; i<34; i++)
  {
    lcd_dat0(gcp64_1[i]);
    lcd_dat0(0x01);
  }

  lcd_cmd(GCP16);
  for (i=0; i<15; i++)
  {
    lcd_dat0(gcp16[i]);
  }

  lcd_cmd(GSSET);
  lcd_dat0(0x00);

  lcd_cmd(OSSEL);
  lcd_dat0(0x00);

  lcd_cmd(SLPOUT);
  
//  _delay_ms(7);

  lcd_cmd(SD_CSET);
  lcd_dat0(0x08);
  lcd_dat0(0x01);
  lcd_dat0(0x8B);
  lcd_dat0(0x01);

  lcd_cmd(SD_PSET);
  lcd_dat0(0x00);
  lcd_dat0(0x8F);


  lcd_cmd(ASCSET);
  lcd_dat0(0x00);
  lcd_dat0(0xAF);
  lcd_dat0(0xAF);
  lcd_dat0(0x03);
  
  lcd_cmd(SCSTART);
  lcd_dat0(0x00);

  lcd_rs_low();
  lcd_dat0(DISON);
  

  lcd_cs_high();  // deselect display 
	

}

void fill_screen(unsigned short color)
{
  unsigned long i;
  backcolor=color;
  lcd_cs_low();  // select display
  
  lcd_cmd(SD_CSET);
  lcd_dat0(0x08);  // start is 8, not 0
  lcd_dat0(0x01);
  lcd_dat0(0x8B);  // is 00x8B = 0x83+8
  lcd_dat0(0x01);

  lcd_cmd(SD_PSET);
  lcd_dat0(0x00);
  lcd_dat0(0xAF);
  
  lcd_cmd(RAMWR);

  // start data transmission  
  for (i=0; i<DISP_W*DISP_H; i++)
    lcd_dat16(color);
    
  lcd_cs_high();  // deselect display    
}


void put_char(unsigned char x, unsigned char y, unsigned char c) 
{
  unsigned char h,ch,p,mask,he;
  x=x*CHAR_W;
  y=y*CHAR_H;
  lcd_cs_low();  // select display
  
  lcd_cmd(SD_CSET);
  lcd_dat0(0x08+x);  // start is 8, not 0
  lcd_dat0(0x01);
  lcd_dat0(0x08+x+CHAR_W-1);  
  lcd_dat0(0x01);
  
  lcd_cmd(SD_PSET);
  lcd_dat0(y);
  if (y < DISP_H-CHAR_H)
  {
    he=CHAR_H;
    lcd_dat0(y+CHAR_H-1);
  }
  else
  {
    he=DISP_H-y;
    lcd_dat0(DISP_H-1);
  }

  lcd_cmd(RAMWR);
  
  for (h=0; h<he; h++) // every column of the character
  {        
        ch=ascii_tab[ c-32 ][h];

        mask=0x80;
        for (p=0; p<CHAR_W; p++)  // write the pixels
        {
          if (ch&mask)
          {
            lcd_dat16(textcolor);                
          }
          else
          {
            lcd_dat16(backcolor);
          }
          mask=mask/2;
        }  // for p
  }
  lcd_cs_high();  // deselect display    
}

void pset(char x, char y, unsigned short col)
{
  lcd_cs_low();  // select display
  
  lcd_cmd(SD_CSET);
  lcd_dat0(0x08+x);  // start is 8, not 0
  lcd_dat0(0x01);
  lcd_dat0(0x08+x);  // end is 00x8B = 0x83+8
  lcd_dat0(0x01);
  
  lcd_cmd(SD_PSET);
  lcd_dat0(y);
  lcd_dat0(y);
  lcd_cmd(RAMWR);
  lcd_dat16(col);  
  lcd_cs_high();  // deselect display    
}
/*
void drawline(char x1, char y1, char x2, char y2, unsigned short color)
{
	char length, xTmp, yTmp, i, y, yAlt;
	short m;
	
	if(x1 == x2) {									// vertical line
		// x1|y1 must be the upper point
		if(y1 > y2) {
			xTmp = x1;
			yTmp = y1;
			x1 = x2;
			y1 = y2;
			x2 = xTmp;
			y2 = yTmp;
		}
		
		length = y2-y1;
		for(i=0; i<=length; i++) {
			pset(x1, y1+i, color);
		}
	} else if(y1 == y2) {							// horizontal line
		// x1|y1 must be the left point
		if(x1 > x2) {
			xTmp = x1;
			yTmp = y1;
			x1 = x2;
			y1 = y2;
			x2 = xTmp;
			y2 = yTmp;
		}
		
		length = x2-x1;
		for(i=0; i<=length; i++) {
			pset(x1+i, y1, color);
		}
	} else {
		// x1 must be smaller than x2
		if(x1 > x2) {
			xTmp = x1;
			yTmp = y1;
			x1 = x2;
			y1 = y2;
			x2 = xTmp;
			y2 = yTmp;
		}
		
		if((y2-y1) >= (x2-x1) || (y1-y2) >= (x2-x1)) {	// angle larger or equal 45
			length = x2-x1;								// not really the length :)
			m = ((y2-y1)*200)/length;
			yAlt = y1;
			for(i=0; i<=length; i++) {
				y = ((m*i)/200)+y1;
				
				if((m*i)%200 >= 100)
					y++;
				else if((m*i)%200 <= -100)
					y--;
				
				drawline(x1+i, yAlt, x1+i, y, color);
				if(length<=(y2-y1) && y1 < y2)
					yAlt = y+1;
				else if(length<=(y1-y2) && y1 > y2)
					yAlt = y-1;
				else
					yAlt = y;
			}
		} else {										// angle smaller 45
			// y1 must be smaller than y2
			if(y1 > y2) {
				xTmp = x1;
				yTmp = y1;
				x1 = x2;
				y1 = y2;
				x2 = xTmp;
				y2 = yTmp;
			}
			length = y2-y1;
			m = ((x2-x1)*200)/length;
			yAlt = x1;
			for(i=0; i<=length; i++) {
				y = ((m*i)/200)+x1;
				
				if((m*i)%200 >= 100)
					y++;
				else if((m*i)%200 <= -100)
					y--;
				
				drawline(yAlt, y1+i, y, y1+i,color);
				if(length<=(x2-x1) && x1 < x2)
					yAlt = y+1;
				else if(length<=(x1-x2) && x1 > x2)
					yAlt = y-1;
				else
					yAlt = y;
			}
		}
	}
} */  /*

void plot (unsigned char *buffer, unsigned long count, unsigned short color)
{
unsigned long cnt;
	for (cnt=1;cnt<count;cnt++){
		drawline( buffer[cnt+1], ((cnt+1)*175/count), buffer[cnt],((cnt*175)/count), color);
	}
}	 */
/*
void displayint2(short data, char x, char y)
  {
    unsigned char byt;
	byt = data / 1000;
	data = data - byt * 1000;
	byt = data / 100;
	data = data - byt * 100;
	byt = data / 10;
	data = data - byt * 10;
	put_char(x, y, byt + 48);
	put_char(x+1, y, data + 48); 
}

void displayint4(short data, char x, char y)
  {
    unsigned char byt;
	byt = data / 1000;
	data = data - byt * 1000;
	put_char(x,y,byt + 48);
	byt = data / 100;
	data = data - byt * 100;
	put_char(x+1,y,byt + 48);
	byt = data / 10;
	data = data - byt * 10;
	put_char(x+2,y,byt + 48);
	put_char(x+3,y,data + 48); 
}*/

void displayint8(long data, char x, char y)
  {
    unsigned char byt;
    byt = data / 10000000;
	data = data - byt * 10000000;
	put_char(x,y,byt + 48);

    byt = data / 1000000;
	data = data - byt * 1000000;
	put_char(x+1,y,byt + 48);

    byt = data / 100000;
	data = data - byt * 100000;
	put_char(x+2,y,byt + 48);

    byt = data / 10000;
	data = data - byt * 10000;
	put_char(x+3,y,byt + 48);

	byt = data / 1000;
	data = data - byt * 1000;
	put_char(x+4,y,byt + 48);

	byt = data / 100;
	data = data - byt * 100;
	put_char(x+5,y,byt + 48);

	byt = data / 10;
	data = data - byt * 10;
	put_char(x+6,y,byt + 48);
	put_char(x+7,y,data + 48); 
}

void displaystr(char *text, char x, char y)
{
	int i=0;
	while(text[i]!=0)
	{
		put_char(x+i,y,text[i]);
		i++;
	}
}
